#include "socktools.h"
#include <string.h>

struct sockaddr_in pop3addr;
short  localport_ho;
unsigned long   localaddr_ho;

/******************************************************************************/
#define POP3_RETR      0x01
#define POP3_DOTTYPE   0x02
#define POP3_STDTYPE   0x03
#define POP3_ERROR     0xff

int checkCommand (const char *cmdstr) {
   if (strncmpi("QUIT", cmdstr, 4) == 0)
      return POP3_STDTYPE;
   else if (strncmpi("APOP", cmdstr, 4) == 0)
      return POP3_STDTYPE;
   else if (strncmpi("USER", cmdstr, 4) == 0)
      return POP3_STDTYPE;
   else if (strncmpi("PASS", cmdstr, 4) == 0)
      return POP3_STDTYPE;
   else if (strncmpi("STAT", cmdstr, 4) == 0)
      return POP3_STDTYPE;
   else if (strncmpi("DELE", cmdstr, 4) == 0)
      return POP3_STDTYPE;
   else if (strncmpi("NOOP", cmdstr, 4) == 0)
      return POP3_STDTYPE;
   else if (strncmpi("RSET", cmdstr, 4) == 0)
      return POP3_STDTYPE;
   //DOT ENDED LIST
   else if (strncmpi("RETR", cmdstr, 4) == 0)
      return POP3_RETR;
   else if (strncmpi("TOP", cmdstr, 3) == 0)
      return POP3_DOTTYPE;
   //A or B ???
   else if (strncmpi("LIST", cmdstr, 4) == 0)
      return ((strlen(cmdstr) > 4) ? POP3_STDTYPE : POP3_DOTTYPE);
   else if (strncmpi("UIDL", cmdstr, 4) == 0)
      return ((strlen(cmdstr) > 4) ? POP3_STDTYPE : POP3_DOTTYPE);

   return POP3_ERROR;
}


/******************************************************************************/
void killTagHelper (char *buf1, char *buf2, const char *uppersub) {
   int   x;
   char *p = NULL;

   while ((p = strstr(buf2, uppersub)) != NULL) {
      x = p - buf2;
      x++;
      buf2[x] = '_';
      buf1[x] = '_';
   }
}


void killExtHelper (char *buf1, char *buf2, char *ptr, const char *uppersub) {
   char *sp;
   int x;
   sp = ptr;
   while ((sp = strstr(sp, uppersub)) != NULL) {
      x = sp - buf2;
      buf1[x] = '_';
      buf2[x] = '_';
   }
}


void washLine(char *buf) {
   int   x;
   char  buf2[2048];
   char *sp = NULL;

   //make upper-case version for quicker searching
   for (x=0; x < strlen(buf); x++) {
      buf2[x] = (char)toupper((int)buf[x]);
   }
   buf2[strlen(buf)] = '\0';

   //Mess-up Evil Tags
   killTagHelper(buf, buf2, "<IFRAME");
   killTagHelper(buf, buf2, "<FRAME");
   killTagHelper(buf, buf2, "<FRAMESET");
   killTagHelper(buf, buf2, "<IMG");
   killTagHelper(buf, buf2, "<LINK");
   killTagHelper(buf, buf2, "<SCRIPT");
   killTagHelper(buf, buf2, "<EMBED");
   killTagHelper(buf, buf2, "<OBJECT");
   killTagHelper(buf, buf2, "<APPLET");
   killTagHelper(buf, buf2, "<BGSOUND");
   killTagHelper(buf, buf2, "</IFRAME>");
   killTagHelper(buf, buf2, "</FRAME>");
   killTagHelper(buf, buf2, "</FRAMESET>");
   killTagHelper(buf, buf2, "</SCRIPT>");
   killTagHelper(buf, buf2, "</EMBED>");
   killTagHelper(buf, buf2, "</OBJECT>");
   killTagHelper(buf, buf2, "</APPLET>");
   //killTagHelper(buf, buf2, "<!--");

   //name / filename
   if ((sp = strstr(buf2, "NAME=")) != NULL)
      sp += 5;
   else if ((sp = strstr(buf2, "FILENAME=")) != NULL)
      sp += 9;

   if (sp != NULL) {
      killExtHelper(buf, buf2, sp, ".EXE");
      killExtHelper(buf, buf2, sp, ".COM");
      killExtHelper(buf, buf2, sp, ".SCR");
      killExtHelper(buf, buf2, sp, ".PIF");
      killExtHelper(buf, buf2, sp, ".LNK");
      killExtHelper(buf, buf2, sp, ".DLL");
      killExtHelper(buf, buf2, sp, ".CPL");
      killExtHelper(buf, buf2, sp, ".BAT");
      killExtHelper(buf, buf2, sp, ".CMD");
      killExtHelper(buf, buf2, sp, ".JS");
      killExtHelper(buf, buf2, sp, ".VBS");
   }
}
/******************************************************************************/




/* ??? */
void handleConn(int sock) {
   char   cbuf[2048] = "";
   int    curcmd;
   int    outsock;

   //-CONNECT-TO-REMOTE-SERVER-------------------------------------------------
   //make socket
   if ((outsock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
      printf("socket() err\n");
      sendLine(sock, "-ERR Server not found");
      return;
   }

   //connect
   if (connect(outsock, (struct sockaddr *)&pop3addr, sizeof(struct sockaddr_in)) < 0) {
      printf("connect() err\n");
      sendLine(sock, "-ERR Server not found");
      closesocket(outsock);
      return;
   }

   //get ok from remote server
   if (recvLine(outsock, cbuf, 2048) == NULL) {
      printf("outsock rx error");
      closesocket(outsock);
      return;
   }
   //send message to client
   if (sendLine(sock, cbuf) < 0 )   {
      closesocket(outsock); return; }


   //-COMMAND-PROXY------------------------------------------------------------
   //get line from client
   while (recvLine(sock, cbuf, 2048) != NULL) {
      printf ("Tx: %s\r\n", cbuf);

      //check type of request
      curcmd = checkCommand(cbuf);

      //Don't understand this command !!!
      if (curcmd == POP3_ERROR) {
         //send response to client
         if (sendLine(sock, "-ERR I don't understand this command !!!") < 0 ) {
            closesocket(outsock); return;                                     }
         continue;
      }

      //send request to remote
      if (sendLine(outsock, cbuf) < 0 ) {
         closesocket(outsock); return;  }

      //get line from remote
      if (recvLine(outsock, cbuf, 2048) == NULL) {
         closesocket(outsock); return;           }

      //send response to client
      if (sendLine(sock, cbuf) < 0 )    {
         closesocket(outsock); return;  }
      printf ("Rx %s\r\n", cbuf);


      //if last command was 'RETR' then your about to recive mail !!!
      if (curcmd == POP3_RETR) {
         while (strcmp(".", cbuf) != 0) {
            //get line from remote
            if (recvLine(outsock, cbuf, 2048) == NULL) {
               closesocket(outsock); return;           }

            //clean line
            washLine(cbuf);

            //send line to client
            if (sendLine(sock, cbuf) < 0 )   {
               closesocket(outsock); return; }
         }
      } else if (curcmd == POP3_DOTTYPE) {
         while (strcmp(".", cbuf) != 0) {
            //get line from remote
            if (recvLine(outsock, cbuf, 2048) == NULL) {
               closesocket(outsock); return;           }
            //send line to client
            if (sendLine(sock, cbuf) < 0 )   {
               closesocket(outsock); return; }
         }
      }
   }
   printf("Bye bye\n");
   closesocket(outsock);
}


/******************************************************************************/
/******************************************************************************/
int haha(int x) {
   switch(x) {
    case 1: //args
      printf("Error: Incorrect number of parameters.\n\n");
      break;
    case 2: //remote host ???
      printf("Error: Could not resolve remote hostname.\n\n");
      break;
    case 3: //remote port
      printf("Error: Invalid remote port (NaN).\n\n");
      break;
    case 4:
      printf("Error: Could not resolve local hostname.\n\n");
      break;
    case 5:
      printf("Error: Invalid local port (NaN).\n\n");
      break;
    default:
      printf("Error: unknowen error.\n\n");
   }
   
   printf("%s\n%s\n\n%s\n%s\n%s\n%s\n%s\n",
   "Usage:",
   "  POP3GUARD.EXE <Remote-IP> <Remote-Port> [<Local-IP> <Local-Port>]",
   "Where:",
   "  <Remote-IP>   : The IP Address of the remote pop3 server.",
   "  <Remote-Port> : The Port of the pop3 server to connect to.",
   "  <Local-IP>    : The IP Address of your machine.            (optional)",
   "  <Local-Port>  : The Port of your machine.                  (optional)"
   );

   return x;
}
/******************************************************************************/
int handleArgs(int argc, char *argv[]) {
   struct hostent *host;

   if ((argc != 3) && (argc != 5)) return haha(1);

   memset(&pop3addr, 0, sizeof(struct sockaddr_in));
   pop3addr.sin_family = AF_INET;


   //Remote IP Address
   if (isdigit(argv[1][0])) {
      pop3addr.sin_addr.s_addr = inet_addr(argv[1]);
   } else {
      if ((host = gethostbyname(argv[1])) == NULL) return haha(2);
      pop3addr.sin_addr.s_addr = *((unsigned long *)host->h_addr_list[0]);
   }

   //Remote IP Port              8282
   if (isdigit(argv[2][0])) {
      pop3addr.sin_port = htons((short)atoi(argv[2]));
   } else {
      return haha(3);
   }


   //local listener addr
   if (argc == 5) {

      //Local IP Address
      if (isdigit(argv[3][0])) {
         localaddr_ho = ntohl(inet_addr(argv[3]));
      } else {
         if ((host = gethostbyname(argv[3])) == NULL) return haha(4);
         localaddr_ho = ntohl(*((unsigned long *)host->h_addr_list[0]));
      }

      //Local IP Port              8282
      if (isdigit(argv[4][0])) {
         localport_ho = (short)atoi(argv[4]);
      } else {
         return haha(5);
      }
   } else {
      localaddr_ho = INADDR_ANY;
      localport_ho = 110;
   }


   return 0;
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

void main (int argc, char *argv[]) {
   int  sock1;
   int  sock2;

   printf("%s\r\n%s\r\n%s\r\n\r\n",
          "POP3 Guard for Win32 (Version 1.00 beta)",
          "Written By: Mark R Hives",
          "WARNING - THIS PROGRAM IS STILL UNDER DEVELOPMENT");

   if (!startWS())  return;

   if (handleArgs(argc, argv) != 0)  return;

   if ((sock1 = serverSocket(localaddr_ho, localport_ho)) < 0) {
      printf("serverSocket() err\n");
      return;
   }

   while (1) {
      sock2 = accept(sock1, NULL, NULL);
      if (sock2 < 0) {
         printf("accept() err\n");
      } else {
         printf("New Connection\n");
         handleConn(sock2);
         closesocket(sock2);
      }
   }

/* --- never reached ---
   closesocket(sock1);
   WSACleanup();         */
}

/*
void old_main_1 (void) {
   char buf1[900];
   int  sock1;
   int  sock2;

   if (!startWS())  return;

   if ((sock1 = serverSocket(INADDR_ANY, 6000)) < 0) {
      printf("serverSocket() err\n");
      return;
   }

   sock2 = accept(sock1, NULL, NULL);
   if (sock2 < 0) {
      printf("accept() err\n");
      closesocket(sock1);
      return;
   }

   while (recvLine(sock2, buf1, 900) != NULL) {
      printf("%i-%s\n", getCmdCode(buf1), buf1);
   }
   printf("--- END ---\n");
   closesocket(sock2);
   closesocket(sock1);
   WSACleanup();
}
*/
